@@ -9,10 +9,10 @@ import maitai
 from maitai._config import config
 from maitai._evaluator import Evaluator
 from maitai._openai_types import Body, Headers, Query
-from maitai._types import EvaluateCallback
 from maitai._utils import convert_open_ai_chat_completion_chunk, convert_openai_chat_completion, get_chat_completion_params, required_args
 from maitai_common.utils.proto_utils import openai_messages_to_proto
-from maitai_gen.chat import ChatCompletionChunk, ChatCompletionResponse, ChatMessage, EvaluationContentType
+from maitai_common.utils.types import EvaluateCallback
+from maitai_gen.chat import ChatCompletionChunk, ChatCompletionParams, ChatCompletionResponse, ChatMessage, EvaluationContentType
 from maitai_gen.config import InferenceLocations
 from maitai_gen.inference import InferenceStreamResponse
 
@@ -59,8 +59,6 @@ class Chat:
 
 class Completions:
     def __init__(self, client: Optional[openai.Client] = None):
-        if client is None:
-            client = openai.OpenAI()
         self.client = client
 
     @required_args(["session_id", "action_type", "application_ref_name", "messages"],
@@ -111,6 +109,8 @@ class Completions:
             raise ValueError("server_side_inference must be true to apply_corrections")
         if apply_corrections is True and evaluation_enabled is False:
             raise ValueError("evaluations must be enabled to apply_corrections")
+        if self.client is None:
+            self.client = openai.OpenAI()
         maitai_config = config.get_application_action_config(application_ref_name, action_type)
         if server_side_inference is None:
             server_side_inference = maitai_config.inference_location == InferenceLocations.SERVER
@@ -134,28 +134,28 @@ class Completions:
             max_tokens = maitai_config.max_tokens
         if n == openai.NOT_GIVEN:
             n = maitai_config.n
+        completion_params = get_chat_completion_params(
+            messages=messages,
+            model=model,
+            frequency_penalty=frequency_penalty,
+            logit_bias=logit_bias,
+            logprobs=logprobs,
+            max_tokens=max_tokens,
+            n=n,
+            presence_penalty=presence_penalty,
+            response_format=response_format,
+            seed=seed,
+            stop=stop,
+            stream=stream,
+            stream_options=stream_options,
+            temperature=temperature,
+            tool_choice=tool_choice,
+            tools=tools,
+            top_logprobs=top_logprobs,
+            top_p=top_p,
+            user=user,
+        )
         if server_side_inference:
-            completion_params = get_chat_completion_params(
-                messages=messages,
-                model=model,
-                frequency_penalty=frequency_penalty,
-                logit_bias=logit_bias,
-                logprobs=logprobs,
-                max_tokens=max_tokens,
-                n=n,
-                presence_penalty=presence_penalty,
-                response_format=response_format,
-                seed=seed,
-                stop=stop,
-                stream=stream,
-                stream_options=stream_options,
-                temperature=temperature,
-                tool_choice=tool_choice,
-                tools=tools,
-                top_logprobs=top_logprobs,
-                top_p=top_p,
-                user=user,
-            )
             response_timeout = None
             if isinstance(timeout, float) or isinstance(timeout, int):
                 response_timeout = timeout
@@ -197,7 +197,7 @@ class Completions:
 
             )
             if stream:
-                return _process_openai_stream(session_id, reference_id, action_type, application_ref_name, messages, response, evaluation_enabled, callback)
+                return _process_openai_stream(session_id, reference_id, action_type, application_ref_name, messages, response, evaluation_enabled, completion_params, callback)
             else:
                 maitai_completion = convert_openai_chat_completion(response)
                 proto_messages = openai_messages_to_proto(messages)
@@ -212,6 +212,19 @@ class Completions:
                         content=proto_messages,
                         application_ref_name=application_ref_name,
                         callback=callback,
+                        chat_completion_response=maitai_completion,
+                        completion_params=completion_params,
+                    )
+                else:
+                    maitai.Inference.store_chat_response(
+                        session_id=session_id,
+                        reference_id=reference_id,
+                        action_type=action_type,
+                        application_ref_name=application_ref_name,
+                        chat_completion_response=maitai_completion,
+                        completion_params=completion_params,
+                        final_chunk=None,
+                        content=""
                     )
                 return maitai_completion
 
@@ -223,7 +236,8 @@ def _process_openai_stream(session_id: Union[str, int],
                            messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
                            stream: openai.Stream[openai_chat_types.ChatCompletionChunk],
                            evaluation_enabled: bool,
-                           callback: Optional[EvaluateCallback] = None) -> Iterable[ChatCompletionChunk]:
+                           chat_completion_params: ChatCompletionParams,
+                           callback: Optional[EvaluateCallback] = None, ) -> Iterable[ChatCompletionChunk]:
     full_body = ""
     proto_messages = openai_messages_to_proto(messages)
     for chunk in stream:
@@ -232,17 +246,31 @@ def _process_openai_stream(session_id: Union[str, int],
         if content is not None:
             full_body += content
         yield maitai_chunk
-        if maitai_chunk.choices[0].finish_reason and evaluation_enabled:
-            proto_messages.append(ChatMessage(role="assistant", content=full_body))
-            Evaluator.evaluate(
-                session_id=session_id,
-                reference_id=reference_id,
-                action_type=action_type,
-                content_type=EvaluationContentType.MESSAGE,
-                content=proto_messages,
-                application_ref_name=application_ref_name,
-                callback=callback
-            )
+        if maitai_chunk.choices[0].finish_reason:
+            if evaluation_enabled:
+                proto_messages.append(ChatMessage(role="assistant", content=full_body))
+                Evaluator.evaluate(
+                    session_id=session_id,
+                    reference_id=reference_id,
+                    action_type=action_type,
+                    content_type=EvaluationContentType.MESSAGE,
+                    content=proto_messages,
+                    application_ref_name=application_ref_name,
+                    callback=callback,
+                    chat_completion_chunk=maitai_chunk,
+                    completion_params=chat_completion_params,
+                )
+            else:
+                maitai.Inference.store_chat_response(
+                    session_id=session_id,
+                    reference_id=reference_id,
+                    action_type=action_type,
+                    application_ref_name=application_ref_name,
+                    completion_params=chat_completion_params,
+                    final_chunk=maitai_chunk,
+                    content=full_body,
+                    chat_completion_response=None,
+                )
             return
 
 
